==> bouml_2.2.2.tar.gz: bouml_2.2.2/src/diagram/DeploymentNodeCanvas.cpp <==
// *************************************************************************
//
// Copyright (C) 2004,2005 Bruno PAGES  All rights reserved.
//
// This file is part of the BOUML Uml Toolkit.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// e-mail : bouml@free.fr
// home   : http://bouml.free.fr
//
// *************************************************************************

#include <qcursor.h>
#include <qpainter.h>
#include <qpopupmenu.h> 

#include "DeploymentNodeCanvas.h"
#include "SimpleRelationCanvas.h"
#include "SimpleData.h"
#include "BrowserDeploymentNode.h"
#include "BrowserDiagram.h"
#include "UmlGlobal.h"
#include "Settings.h"
#include "SettingsDialog.h"
#include "InstanceDialog.h"
#include "myio.h"
#include "ToolCom.h"
#include "Tool.h"
#include "MenuTitle.h"

DeploymentNodeCanvas::DeploymentNodeCanvas(BrowserNode * bn, UmlCanvas * canvas,
					   int x, int y, int id)
    : DiagramCanvas(0, canvas, x, y, DEPLOYMENTNODE_CANVAS_MIN_SIZE,
		    DEPLOYMENTNODE_CANVAS_MIN_SIZE, id) {
  browser_node = bn;
  itscolor = UmlDefaultColor;
  write_horizontally = UmlDefaultState;
  check_size();
  connect(DrawingSettings::instance(), SIGNAL(changed()), this, SLOT(modified()));
  connect(bn->get_data(), SIGNAL(changed()), this, SLOT(modified()));
  connect(bn->get_data(), SIGNAL(deleted()), this, SLOT(deleted()));
}

DeploymentNodeCanvas::~DeploymentNodeCanvas() {
}

void DeploymentNodeCanvas::delete_it() {
  disconnect(DrawingSettings::instance(), SIGNAL(changed()), this, SLOT(modified()));
  disconnect(browser_node->get_data(), 0, this, 0);
  DiagramCanvas::delete_it();
}

void DeploymentNodeCanvas::deleted() {
  delete_it();
  canvas()->update();
}

const QString & DeploymentNodeCanvas::get_name() const {
  return iname;
}

void DeploymentNodeCanvas::set_name(const QString & s) {
  iname = s;
}

BrowserNode * DeploymentNodeCanvas::get_type() const {
  return browser_node;
}

BrowserNodeList DeploymentNodeCanvas::get_types() const {
  return BrowserDeploymentNode::instances();
}

void DeploymentNodeCanvas::set_type(BrowserNode * t) {
  if (browser_node != t) {
    disconnect(browser_node->get_data(), SIGNAL(changed()), this, SLOT(modified()));
    disconnect(browser_node->get_data(), SIGNAL(deleted()), this, SLOT(deleted()));
    browser_node = t;
    connect(browser_node->get_data(), SIGNAL(changed()), this, SLOT(modified()));
    connect(browser_node->get_data(), SIGNAL(deleted()), this, SLOT(deleted()));
  }
}

int DeploymentNodeCanvas::min_width() {
  QFontMetrics fm(the_canvas()->get_font(UmlNormalFont));
  
  int wi;
  
  if (horiz)
    wi = fm.width(iname + ":" + browser_node->get_name());
  else {
    wi = fm.width(browser_node->get_name());
    
    int iw = fm.width(iname + ":");
    
    if (iw > wi)
      wi = iw;
  }
  
  const char * st = browser_node->get_data()->get_stereotype();
  
  if (st[0]) {
    int stw = fm.width(QString("<<") + st + ">>");
    
    if (wi < stw)
      wi = stw;
  }
  
  wi += DEPLOYMENTNODE_CANVAS_ADDED + (int) (8 * the_canvas()->zoom());
  
  return (wi < DEPLOYMENTNODE_CANVAS_MIN_SIZE)
    ? DEPLOYMENTNODE_CANVAS_MIN_SIZE
    : wi;
}

int DeploymentNodeCanvas::min_height() {
  QFontMetrics fm(the_canvas()->get_font(UmlNormalFont));
  int two = (int) (2 * the_canvas()->zoom());
  int h = fm.height() + two;
  const char * st = browser_node->get_data()->get_stereotype();
  int he = h;
  
  if (st[0]) 
    he += h;
	    
  if (! horiz)
    he += h;

  return he + DEPLOYMENTNODE_CANVAS_ADDED + two;
}

void DeploymentNodeCanvas::check_size() {
  DeploymentDiagramSettings dflt;
  
  dflt.write_horizontally = write_horizontally;
  the_canvas()->browser_diagram()->get_deploymentdiagramsettings(dflt);
  horiz = (dflt.write_horizontally == UmlYes);
  
  int wi = min_width();
  int he = min_height();
  
  DiagramCanvas::resize((width() < wi) ? wi : width(),
			(height() < he) ? he : height());
}

void DeploymentNodeCanvas::draw(QPainter & p) {
  if (! visible()) return;
  
  QBrush brsh = p.brush();
  QColor bckgrnd = p.backgroundColor();
  UmlColor c = (itscolor != UmlDefaultColor)
    ? itscolor
    : the_canvas()->browser_diagram()->get_color(UmlDeploymentNode);
  QColor co = color(c);
  const int added = (int) (DEPLOYMENTNODE_CANVAS_ADDED * the_canvas()->zoom());
  QRect r = rect();
  QPointArray a(7);
  
  r.setTop(r.top() + added);
  r.setRight(r.right() - added);
  
  a.setPoint(0, r.left(), r.top());
  a.setPoint(1, r.left() + added, r.top() - added);
  a.setPoint(2, r.right() + added, r.top() - added);
  a.setPoint(3, r.right() + added, r.bottom() - added);
  a.setPoint(4, r.right(), r.bottom());
  a.setPoint(5, r.right(), r.top());
  a.setPoint(6, r.left(), r.top());
  
  if (c == UmlWhite) {
    p.setBackgroundMode(QObject::TransparentMode);
    p.setBackgroundColor(co);
    p.drawPolyline(a);
  }
  else {
    p.setBackgroundMode(QObject::OpaqueMode);
    p.fillRect(r, co);
    p.setBrush(co);
    p.drawPolygon(a, TRUE, 0, 6);
    p.setBrush(brsh);
    p.setBackgroundColor(co);
  }

  p.drawRect(r);
  p.drawLine(r.topRight(), a.point(2));

  const int two = (int) (2 * the_canvas()->zoom());  
  QFontMetrics fm(the_canvas()->get_font(UmlNormalFont));
    
  r.setTop(r.top() + two);
  
  p.setFont(the_canvas()->get_font(UmlNormalFont));
  
  if (horiz)
    p.drawText(r, QObject::AlignHCenter + QObject::AlignTop,
	       iname + ":" + browser_node->get_name());
  else {
    p.drawText(r, QObject::AlignHCenter + QObject::AlignTop,
	       iname + ":");
    r.setTop(r.top() + fm.height() + two);
    p.drawText(r, QObject::AlignHCenter + QObject::AlignTop,
	       browser_node->get_name());
  }
  
  const char * st = browser_node->get_data()->get_stereotype();
  
  if (st[0]) {
    r.setTop(r.top() + fm.height() + two);
    p.drawText(r, QObject::AlignHCenter + QObject::AlignTop, 
	       QString("<<") + st + ">>");
  }
  
  p.setBackgroundColor(bckgrnd);
  
  if (selected())
    show_mark(p, rect());
}

UmlCode DeploymentNodeCanvas::type() const {
  return UmlDeploymentNode;
}

void DeploymentNodeCanvas::delete_available(bool & in_model, bool & out_model) const {
  out_model |= TRUE;
  in_model |= browser_node->is_writable();
}

bool DeploymentNodeCanvas::alignable() const {
  return TRUE;
}

bool DeploymentNodeCanvas::copyable() const {
  return selected();
}

void DeploymentNodeCanvas::open() {
  InstanceDialog d(this, QCursor::pos(), "node : ", "Node instance dialog");
  
  if (d.exec() == QDialog::Accepted)
    modified();
}

void DeploymentNodeCanvas::modified() {
  // force son reaffichage
  hide();
  check_size();
  show();
  canvas()->update();
  package_modified();
}

void DeploymentNodeCanvas::menu(const QPoint&) {
  QPopupMenu m(0);
  QPopupMenu toolm(0);
  
  m.insertItem(new MenuTitle(browser_node->get_name(), m.font()), -1);
  m.insertSeparator();
  m.insertItem("upper", 0);
  m.insertItem("lower", 1);
  m.insertSeparator();
  m.insertItem("edit", 2);
  m.insertSeparator();
  m.insertItem("edit drawing settings", 3);
  m.insertSeparator();
  m.insertItem("select node in browser", 4);
  if (linked())
    m.insertItem("select linked items", 5);
  m.insertSeparator();
  m.insertItem("set node associated diagram", 6);
  m.insertSeparator();
  m.insertItem("remove from view", 7);
  m.insertSeparator();
  if (Tool::menu_insert(&toolm, UmlDeploymentNode, 10))
    m.insertItem("tool", &toolm);

  int rank = m.exec(QCursor::pos());
  
  switch (rank) {
  case 0:
    upper();
    modified();	// call package_modified()
    return;
  case 1:
    lower();
    modified();	// call package_modified()
    return;
  case 2:
    open();
    return;
  case 3:
    {
      QArray<StateSpec> st(1);
      QArray<ColorSpec> co(1);
      
      st[0].set("write node instance \nhorizontally", &write_horizontally);
      co[0].set("Node color", &itscolor);

      SettingsDialog dialog(&st, &co, FALSE, TRUE);
      
      if (dialog.exec() == QDialog::Accepted)
	modified();
      return;
    }
    break;
  case 4:
    browser_node->select_in_browser();
    return;
  case 5:
    the_canvas()->unselect_all();
    select_associated();
    return;
  case 6:
    ((BrowserDeploymentNode *) browser_node)
      ->set_associated_diagram((BrowserDeploymentDiagram *)
			       the_canvas()->browser_diagram());
    return;
  case 7:
    // remove from view
    delete_it();
    break;
  default:
    if (rank >= 10)
      ToolCom::run(Tool::command(rank - 10), browser_node);
    return;
  }
  
  package_modified();
}

bool DeploymentNodeCanvas::may_start(UmlCode & l) const {
  switch (l) {
  case UmlDependency:
    l = UmlDependOn;
    return browser_node->is_writable();
  case UmlAssociation:
  case UmlAnchor:
    return TRUE;
  default:
    return FALSE;
  }
}

bool DeploymentNodeCanvas::may_connect(UmlCode l, const DiagramItem * dest) const {
  switch (dest->type()) {
  case UmlDeploymentNode:
    return (l != UmlAnchor);
  case UmlNote:
  case UmlIcon:
    return (l == UmlAnchor);
  case UmlHub:
    return (l == UmlAssociation);
  default:
    return FALSE;
  }
}

void DeploymentNodeCanvas::connexion(UmlCode action, DiagramItem * dest,
				     const QPoint &, const QPoint &) {
  if (action == UmlDependOn)
    (new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0))->show();
  else
    (new ArrowCanvas(the_canvas(), this, dest, action, 0))->show();
}

aCorner DeploymentNodeCanvas::on_resize_point(const QPoint & p) {
  return ::on_resize_point(p, rect());
}

void DeploymentNodeCanvas::resize(aCorner c, int dx, int dy) {
  DiagramCanvas::resize(c, dx, dy, min_width(), min_height());
}

void DeploymentNodeCanvas::save(QTextStream & st, bool ref, QString & warning) const {
  if (ref)
    st << "deploymentnodecanvas_ref " << get_ident() << " // " << browser_node->get_name();
  else {
    nl_indent(st);
    st << "deploymentnodecanvas " << get_ident() << ' ';
    indent(+1);
    browser_node->save(st, TRUE, warning);
    if (!iname.isEmpty()) {
      nl_indent(st);
      st << "name ";
      save_string(iname, st);  
    }
    nl_indent(st);
    if (write_horizontally != UmlDefaultState)
      st << "write_horizontally " << stringify(write_horizontally) << ' ';
    if (itscolor != UmlDefaultColor)
      st << "color " << stringify(itscolor) << ' ';
    save_xyzwh(st, this, "xyzwh");
    indent(-1);
  }
}

DeploymentNodeCanvas * DeploymentNodeCanvas::read(char * & st, UmlCanvas * canvas, char * k) {
  if (!strcmp(k, "deploymentnodecanvas_ref"))
    return ((DeploymentNodeCanvas *) dict_get(read_id(st), "deploymentnodecanvas", canvas));
  else if (!strcmp(k, "deploymentnodecanvas")) {
    int id = read_id(st);
    BrowserNode * br = BrowserDeploymentNode::read_ref(st);
    QString iname;
    Uml3States write_horizontally = UmlDefaultState;
    
    k = read_keyword(st);
    
    if (!strcmp(k, "name")) {
      iname = read_string(st);
      k = read_keyword(st);
    }
    if (!strcmp(k, "write_horizontally") ||
	!strcmp(k, "write_horizontaly")) {
      write_horizontally = state(read_keyword(st));
      k = read_keyword(st);
    }
    
    UmlColor color = UmlDefaultColor;
    
    read_color(st, "color", color, k);
    
    if (strcmp(k, "xyzwh"))
      wrong_keyword(k, "xyzwh");
    
    int x = (int) read_double(st);
    DeploymentNodeCanvas * result =
      new DeploymentNodeCanvas(br, canvas, x, (int) read_double(st), id);
    
    result->iname = iname;
    result->write_horizontally = write_horizontally;
    result->itscolor = color;
    read_zwh(st, result);
    result->check_size();

    result->show();
    return result;
  }
  else
    return 0;
}

==> bouml_2.7.4.tar.gz: bouml_2.7.4/src/diagram/DeploymentNodeCanvas.cpp <==
// *************************************************************************
//
// Copyright (C) 2004,2005 Bruno PAGES  All rights reserved.
//
// This file is part of the BOUML Uml Toolkit.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// e-mail : bouml@free.fr
// home   : http://bouml.free.fr
//
// *************************************************************************

#include <qcursor.h>
#include <qpainter.h>
#include <qpopupmenu.h> 

#include "DeploymentNodeCanvas.h"
#include "SimpleRelationCanvas.h"
#include "SimpleData.h"
#include "BrowserDeploymentNode.h"
#include "BrowserDiagram.h"
#include "UmlGlobal.h"
#include "Settings.h"
#include "SettingsDialog.h"
#include "InstanceDialog.h"
#include "myio.h"
#include "ToolCom.h"
#include "Tool.h"
#include "MenuTitle.h"

DeploymentNodeCanvas::DeploymentNodeCanvas(BrowserNode * bn, UmlCanvas * canvas,
					   int x, int y, int id)
    : DiagramCanvas(0, canvas, x, y, DEPLOYMENTNODE_CANVAS_MIN_SIZE,
		    DEPLOYMENTNODE_CANVAS_MIN_SIZE, id) {
  browser_node = bn;
  itscolor = UmlDefaultColor;
  write_horizontally = UmlDefaultState;
  check_size();
  connect(DrawingSettings::instance(), SIGNAL(changed()), this, SLOT(modified()));
  connect(bn->get_data(), SIGNAL(changed()), this, SLOT(modified()));
  connect(bn->get_data(), SIGNAL(deleted()), this, SLOT(deleted()));
}

DeploymentNodeCanvas::~DeploymentNodeCanvas() {
}

void DeploymentNodeCanvas::delete_it() {
  disconnect(DrawingSettings::instance(), SIGNAL(changed()), this, SLOT(modified()));
  disconnect(browser_node->get_data(), 0, this, 0);
  DiagramCanvas::delete_it();
}

void DeploymentNodeCanvas::deleted() {
  delete_it();
  canvas()->update();
}

const QString & DeploymentNodeCanvas::get_name() const {
  return iname;
}

void DeploymentNodeCanvas::set_name(const QString & s) {
  iname = s;
}

BrowserNode * DeploymentNodeCanvas::get_type() const {
  return browser_node;
}

BrowserNodeList DeploymentNodeCanvas::get_types() const {
  return BrowserDeploymentNode::instances();
}

void DeploymentNodeCanvas::set_type(BrowserNode * t) {
  if (browser_node != t) {
    disconnect(browser_node->get_data(), SIGNAL(changed()), this, SLOT(modified()));
    disconnect(browser_node->get_data(), SIGNAL(deleted()), this, SLOT(deleted()));
    browser_node = t;
    connect(browser_node->get_data(), SIGNAL(changed()), this, SLOT(modified()));
    connect(browser_node->get_data(), SIGNAL(deleted()), this, SLOT(deleted()));
  }
}

int DeploymentNodeCanvas::min_width() {
  QFontMetrics fm(the_canvas()->get_font(UmlNormalFont));
  
  int wi;
  
  if (horiz)
    wi = fm.width(iname + ":" + browser_node->get_name());
  else {
    wi = fm.width(browser_node->get_name());
    
    int iw = fm.width(iname + ":");
    
    if (iw > wi)
      wi = iw;
  }
  
  const char * st = browser_node->get_data()->get_stereotype();
  
  if (st[0]) {
    int stw = fm.width(QString("<<") + st + ">>");
    
    if (wi < stw)
      wi = stw;
  }
  
  wi += DEPLOYMENTNODE_CANVAS_ADDED + (int) (8 * the_canvas()->zoom());
  
  return (wi < DEPLOYMENTNODE_CANVAS_MIN_SIZE)
    ? DEPLOYMENTNODE_CANVAS_MIN_SIZE
    : wi;
}

int DeploymentNodeCanvas::min_height() {
  QFontMetrics fm(the_canvas()->get_font(UmlNormalFont));
  int two = (int) (2 * the_canvas()->zoom());
  int h = fm.height() + two;
  const char * st = browser_node->get_data()->get_stereotype();
  int he = h;
  
  if (st[0]) 
    he += h;
	    
  if (! horiz)
    he += h;

  return he + DEPLOYMENTNODE_CANVAS_ADDED + two;
}

void DeploymentNodeCanvas::check_size() {
  DeploymentDiagramSettings dflt;
  
  dflt.write_horizontally = write_horizontally;
  the_canvas()->browser_diagram()->get_deploymentdiagramsettings(dflt);
  horiz = (dflt.write_horizontally == UmlYes);
  
  int wi = min_width();
  int he = min_height();
  
  if ((width() < wi) || (height() < he))
    DiagramCanvas::resize((width() < wi) ? wi : width(),
			  (height() < he) ? he : height());
}

void DeploymentNodeCanvas::draw(QPainter & p) {
  if (! visible()) return;
  
  QBrush brsh = p.brush();
  QColor bckgrnd = p.backgroundColor();
  UmlColor c = (itscolor != UmlDefaultColor)
    ? itscolor
    : the_canvas()->browser_diagram()->get_color(UmlDeploymentNode);
  QColor co = color(c);
  const int added = (int) (DEPLOYMENTNODE_CANVAS_ADDED * the_canvas()->zoom());
  QRect r = rect();
  QPointArray a(7);
  
  r.setTop(r.top() + added);
  r.setRight(r.right() - added);
  
  a.setPoint(0, r.left(), r.top());
  a.setPoint(1, r.left() + added, r.top() - added);
  a.setPoint(2, r.right() + added, r.top() - added);
  a.setPoint(3, r.right() + added, r.bottom() - added);
  a.setPoint(4, r.right(), r.bottom());
  a.setPoint(5, r.right(), r.top());
  a.setPoint(6, r.left(), r.top());
  
  if (c == UmlWhite) {
    p.setBackgroundMode(QObject::TransparentMode);
    p.setBackgroundColor(co);
    p.drawPolyline(a);
  }
  else {
    p.setBackgroundMode(QObject::OpaqueMode);
    p.fillRect(r, co);
    p.setBrush(co);
    p.drawPolygon(a, TRUE, 0, 6);
    p.setBrush(brsh);
    p.setBackgroundColor(co);
  }

  p.drawRect(r);
  p.drawLine(r.topRight(), a.point(2));

  const int two = (int) (2 * the_canvas()->zoom());  
  QFontMetrics fm(the_canvas()->get_font(UmlNormalFont));
    
  r.setTop(r.top() + two);
  
  p.setFont(the_canvas()->get_font(UmlNormalFont));
  
  if (horiz)
    p.drawText(r, QObject::AlignHCenter + QObject::AlignTop,
	       iname + ":" + browser_node->get_name());
  else {
    p.drawText(r, QObject::AlignHCenter + QObject::AlignTop,
	       iname + ":");
    r.setTop(r.top() + fm.height() + two);
    p.drawText(r, QObject::AlignHCenter + QObject::AlignTop,
	       browser_node->get_name());
  }
  
  const char * st = browser_node->get_data()->get_stereotype();
  
  if (st[0]) {
    r.setTop(r.top() + fm.height() + two);
    p.drawText(r, QObject::AlignHCenter + QObject::AlignTop, 
	       QString("<<") + st + ">>");
  }
  
  p.setBackgroundColor(bckgrnd);
  
  if (selected())
    show_mark(p, rect());
}

UmlCode DeploymentNodeCanvas::type() const {
  return UmlDeploymentNode;
}

void DeploymentNodeCanvas::delete_available(bool & in_model, bool & out_model) const {
  out_model |= TRUE;
  in_model |= browser_node->is_writable();
}

bool DeploymentNodeCanvas::alignable() const {
  return TRUE;
}

bool DeploymentNodeCanvas::copyable() const {
  return selected();
}

void DeploymentNodeCanvas::open() {
  InstanceDialog d(this, QCursor::pos(), "node : ", "Node instance dialog");
  
  if (d.exec() == QDialog::Accepted)
    modified();
}

void DeploymentNodeCanvas::modified() {
  // force son reaffichage
  hide();
  check_size();
  show();
  canvas()->update();
  package_modified();
}

void DeploymentNodeCanvas::menu(const QPoint&) {
  QPopupMenu m(0);
  QPopupMenu toolm(0);
  
  m.insertItem(new MenuTitle(browser_node->get_name(), m.font()), -1);
  m.insertSeparator();
  m.insertItem("upper", 0);
  m.insertItem("lower", 1);
  m.insertSeparator();
  m.insertItem("edit", 2);
  m.insertSeparator();
  m.insertItem("edit drawing settings", 3);
  m.insertSeparator();
  m.insertItem("select node in browser", 4);
  if (linked())
    m.insertItem("select linked items", 5);
  m.insertSeparator();
  m.insertItem("set node associated diagram", 6);
  m.insertSeparator();
  m.insertItem("remove from view", 7);
  m.insertSeparator();
  if (Tool::menu_insert(&toolm, UmlDeploymentNode, 10))
    m.insertItem("tool", &toolm);

  int rank = m.exec(QCursor::pos());
  
  switch (rank) {
  case 0:
    upper();
    modified();	// call package_modified()
    return;
  case 1:
    lower();
    modified();	// call package_modified()
    return;
  case 2:
    open();
    return;
  case 3:
    {
      QArray<StateSpec> st(1);
      QArray<ColorSpec> co(1);
      
      st[0].set("write node instance \nhorizontally", &write_horizontally);
      co[0].set("Node color", &itscolor);

      SettingsDialog dialog(&st, &co, FALSE, TRUE);
      
      if (dialog.exec() == QDialog::Accepted)
	modified();
      return;
    }
    break;
  case 4:
    browser_node->select_in_browser();
    return;
  case 5:
    the_canvas()->unselect_all();
    select_associated();
    return;
  case 6:
    ((BrowserDeploymentNode *) browser_node)
      ->set_associated_diagram((BrowserDeploymentDiagram *)
			       the_canvas()->browser_diagram());
    return;
  case 7:
    // remove from view
    delete_it();
    break;
  default:
    if (rank >= 10)
      ToolCom::run(Tool::command(rank - 10), browser_node);
    return;
  }
  
  package_modified();
}

const char * DeploymentNodeCanvas::may_start(UmlCode & l) const {
  switch (l) {
  case UmlDependency:
    l = UmlDependOn;
    return (browser_node->is_writable()) ? 0 : "read only";
  case UmlAssociation:
  case UmlAnchor:
    return 0;
  default:
    return "illegal";
  }
}

const char * DeploymentNodeCanvas::may_connect(UmlCode l, const DiagramItem * dest) const {
  switch (dest->type()) {
  case UmlDeploymentNode:
    return (l != UmlAnchor) ? 0 : "can't have an anchor between nodes";
  case UmlNote:
  case UmlIcon:
    return (l == UmlAnchor) ? 0 : "illegal";
  case UmlHub:
    return (l == UmlAssociation) ? 0 : "illegal";
  default:
    return "illegal";
  }
}

void DeploymentNodeCanvas::connexion(UmlCode action, DiagramItem * dest,
				     const QPoint &, const QPoint &) {
  if (action == UmlDependOn)
    (new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0))->show();
  else
    (new ArrowCanvas(the_canvas(), this, dest, action, 0))->show();
}

aCorner DeploymentNodeCanvas::on_resize_point(const QPoint & p) {
  return ::on_resize_point(p, rect());
}

void DeploymentNodeCanvas::resize(aCorner c, int dx, int dy) {
  DiagramCanvas::resize(c, dx, dy, min_width(), min_height());
}

void DeploymentNodeCanvas::save(QTextStream & st, bool ref, QString & warning) const {
  if (ref)
    st << "deploymentnodecanvas_ref " << get_ident() << " // " << browser_node->get_name();
  else {
    nl_indent(st);
    st << "deploymentnodecanvas " << get_ident() << ' ';
    indent(+1);
    browser_node->save(st, TRUE, warning);
    if (!iname.isEmpty()) {
      nl_indent(st);
      st << "name ";
      save_string(iname, st);  
    }
    nl_indent(st);
    if (write_horizontally != UmlDefaultState)
      st << "write_horizontally " << stringify(write_horizontally) << ' ';
    if (itscolor != UmlDefaultColor)
      st << "color " << stringify(itscolor) << ' ';
    save_xyzwh(st, this, "xyzwh");
    indent(-1);
  }
}

DeploymentNodeCanvas * DeploymentNodeCanvas::read(char * & st, UmlCanvas * canvas, char * k) {
  if (!strcmp(k, "deploymentnodecanvas_ref"))
    return ((DeploymentNodeCanvas *) dict_get(read_id(st), "deploymentnodecanvas", canvas));
  else if (!strcmp(k, "deploymentnodecanvas")) {
    int id = read_id(st);
    BrowserNode * br = BrowserDeploymentNode::read_ref(st);
    QString iname;
    Uml3States write_horizontally = UmlDefaultState;
    
    k = read_keyword(st);
    
    if (!strcmp(k, "name")) {
      iname = read_string(st);
      k = read_keyword(st);
    }
    if (!strcmp(k, "write_horizontally") ||
	!strcmp(k, "write_horizontaly")) {
      write_horizontally = state(read_keyword(st));
      k = read_keyword(st);
    }
    
    UmlColor color = UmlDefaultColor;
    
    read_color(st, "color", color, k);
    
    if (strcmp(k, "xyzwh"))
      wrong_keyword(k, "xyzwh");
    
    int x = (int) read_double(st);
    DeploymentNodeCanvas * result =
      new DeploymentNodeCanvas(br, canvas, x, (int) read_double(st), id);
    
    result->iname = iname;
    result->write_horizontally = write_horizontally;
    result->itscolor = color;
    read_zwh(st, result);
    result->check_size();
    result->set_center100();

    result->show();
    return result;
  }
  else
    return 0;
}
